(* 

   This header is a port of Microsoft's C header file for
   Help DLLs as published in the Help Applications Generator
   authoring guide (the "HAG"). It contains a number of additions
   based on discoveries by a number of authors, additional commentary
   and a critical correction to the definition of the HE_GETINFO
   function abstraction. This header may be used interchangeably for
   both 16 and 32 bit Help. Among those responsible for the additions,
   comments and corrections include Bernd Herd, Cub Lea, Jim Mischel,
   and Matthew Withopf. A few of Matthew's comments remain here
   untranslated, and additional info on some functions is available
   in the HAG docs (downloadable from Microsoft and www.shareware.com
   via a partfilename search on WHAG or WHAT6).

   Several versions of the WinHelp header file created for use with
   Pascal environments have been found to be faulty in some areas.
   This version has been fairly well tested with Delphi, Delphi 2
   and Delphi 3 with baggage and embeddings.

   Cub Lea (9/97)

*)

{unit Hlpdll16;}
unit Hlpdll32;
interface

Uses {$IFDEF Win32}Windows{$ELSE}Wintypes{$ENDIF};

{Macro error structure and error code definitions }
Const
  wMACRO_ERROR  = 128;  { Maximum error msg length }
{Macro error constants }
  wMERR_None    = 0;    { No error }
  wMERR_Memory  = 1;    { Out of memory (local) }
  wMERR_Param   = 2;    { Invalid parameter passed }
  wMERR_File    = 3;    { Invalid file parameter }
  wMERR_Error   = 4;    { General macro error }
  wMERR_Message = 5;    { Macro error with message }
{Flags constants indicating how error may be handled }
  fwMERR_Abort    = 1;  { Allow the "abort" option }
  fwMERR_Continue = 2;  { Allow the "continue" option }
  fwMERR_Retry    = 4;  { Allow the "retry" option }

Type
  pME = ^ME;
  ME = record
	fwFlags : Word;{ Determines how error will be handled }
	wError : Word;{ Macro error code }
	{ Error message (if wError = wMERR_Message) }
	rgchError : Array [0..wMacro_Error-1] of Char;
  End;

{Message classes and message definitions for LDLLHandler }
Const
{Classes of messages that may be sent to DLLs}
  DC_MinMax     = 1;
  DC_InitTerm   = 2;
  DC_Jump       = 4;
  DC_Activate   = 8;
  DC_CallBacks  = 16;

{ -> mgliche Antworten auf DW_WHATMSG-Nachricht }
{Messages sent to DLLs}
  DW_NotUsed   = 0;
  DW_WhatMsg   = 1;
  DW_MinMax    = 2;       { 1=Maximiz/2=Minimiz     -         }
  DW_Size      = 3;       { Lo=MaxX/Hi=MaxY         -         }
  DW_Init      = 4;
  DW_Term      = 5;
  DW_StartJump = 6;
  DW_EndJump   = 7;       { TopicOffset   ScrollBox Position  }
  DW_ChgFile   = 8;       { ^FileName               -         }
  DW_Activate  = 9;       { If 0 Then current has Lost Focus  }
						  { If <>0 then current Has Focus     }
  DW_CallBacks = 10;      { ^FuncTable                        }
  DW_Unknown1  = 11;      { Reported by Bernd Herd            }

{Embedded window messages and structures}
Const {Embedded Window messages}
  EWM_Render         = $706A;
  EWM_QuerySize      = $706B;
  EWM_AskPalette     = $706C;
  EWM_FindNewpalette = $706D;

Const{Function ID constants for use by FAPI}
  HELP_SetContents = $0066;
  HELP_JumpHash    = $0095;
  HELP_PopupHash   = $0096;
  HELP_JmpId       = $0103;
  HELP_PopupId     = $0104;
  HELP_GotoMark    = $0202;

Type
{Embedded Window create structure}
  pEWData = ^TEWData;
  QCI = ^TEWData;
  TEWData = Record
	idMajVersion : Integer;     {unused}
	idMinVersion : Integer;     {unused}
	szFileName : PChar;         {Current Help file}
	szAuthorData : PChar;       {Text passed by the author}
	hfs : THandle;              {Handle to the current file system}
	coFore : LongInt;           {Foreground color for this topic}
	coBack : Longint;           {Background color for this topic}
  End;

{Embedded window rendering info}

  QRI = ^TRenderInfo;
  pRenderInfo = ^TRenderInfo;
  TRenderInfo = Record
	Rect : TRect;
	DC : hDC;{set up for using same as standard Delphi rects/dc's}
  End;

{Constants and type definitions for WinHelp internal functions}
Const
{file mode flags (help file system)}
  fFSReadOnly = 1;           {file (FS) is readonly}
  fFSOpenReadOnly = 2;       {file (FS) will open in readonly mode}
  fFSReadWrite = 0;          {File (FS) is read/write}
  fFSOpenReadWrite = 0;      {file (FS) is opened in read/write mode}
{seek origins}
  wFSSeekSet = 0;
  wFSSeekCur = 1;
  wFSSeekEnd = 2;
{low level info options}
  wLLSameFid = 0;
  wLLDupFid  = 1;
  wLLNewFid  = 2;
{Return codes}
  rcSuccess       = 0;
  rcFailure       = 1;
  rcExists        = 2;
  rcNoExists      = 3;
  rcInvalid       = 4;
  rcBadHandle     = 5;
  rcBadArg        = 6;
  rcUnimplemented = 7;
  rcOutOfMemory   = 8;
  rcNoPermission  = 9;
  rcBadVersion    = 10;
  rcDiskFull      = 11;
  rcInternal      = 12;
  rcNoFileHandles = 13;
  rcFileChange    = 14;
  rcTooBig        = 15;
{following not from core engine:}
  rcReadError = 101;

{Error constants and values for Error}
  wERRS_OOM          = 2;       {Out of memory}
  wERRS_NoHelpPS     = 3;       { No help during printer setup }
  wERRS_NoHelpPR     = 4;       { No help while printing }
  wERRS_FNF          = 1001;    { Cannot find file }
  wERRS_NoTopic      = 1002;    { Topic does not exist }
  wERRS_NoPrinter    = 1003;    { Cannot print }
  wERRS_Print        = 1004;    { Printing error }
  wERRS_Export       = 1005;    { Cannot copy to clipboard }
  wERRS_BadFile      = 1006;    { Not a Windows Help file }
  wERRS_OldFile      = 1007;    { Can't read help file }
  wERRS_Virus        = 1011;    { Bad .EXE }
  wERRS_BadDrive     = 1012;    { Invalid drive }
  wERRS_WinClass     = 1014;    { Bad window class }
  wERRS_BadKeyWord   = 3012;    { Invalid keyword }
  wERRS_BadPathSpec  = 3015;    { Invalid path specification }
  wERRS_PathNotFound = 3017;    { Path not found }
  wERRS_DialogBoxOOM = 3018;    { Insufficient memory for dialog }
  wERRS_DiskFull     = 5001;    { Disk is full }
  wERRS_FSReadWrite  = 5002;    { File read/write failure }

{Actions for LGetInfo}
  GI_Nothing  = 0;              { Not used }
  GI_Instance = 1;              { Application instance handle }
  GI_MainHWnd = 2;              { Main window handle }
  GI_CurrHWnd = 3;              { Current window handle }
  GI_HFS      = 4;              { Handle to file system in use }
  GI_FGColor  = 5;              { Foreground color used by app }
  GI_BKColor  = 6;              { Background color used by app }
  GI_TopicNo  = 7;              { Topic number }
  GI_HPath    = 8;              { Handle containing path }

{Callback Function Table offsets}
  HE_NotUsed            =   0;
  HE_HfsOpenSz          =   1;
  HE_RcCloseHfs         =   2;
  HE_HfOpenHfs          =   3;
  HE_RcCloseHf          =   4;
  HE_LcbReadHf          =   5;
  HE_LTellHf            =   6;
  HE_LSeekHf            =   7;
  HE_FEofHf             =   8;
  HE_LcbSizeHf          =   9;
  HE_FAccessHfs         =  10;
  HE_RcLLInfoFromHf     =  11;
  HE_RcLLInfoFromHfs    =  12;
  HE_ErrorW             =  13;
  HE_ErrorSz            =  14;
  HE_GetInfo            =  15;
  HE_API                =  16;
  HE_LcbSizeHfs         =  17;
  HE_HfCreateFileHfs    =  18;
  HE_RcUnlinkFileHfs    =  19;
  HE_RcFlushHf          =  20;
  HE_LcbWriteHf         =  21;
  HE_RcRenameFileHfs    =  22;
  HE_RcAbandonHf        =  23;
  HE_HfsCreateFileSys   =  24;
  HE_RcDestroyFileSys   =  25;

Type
  RC = Word;            {Error return code}
  HFS = THandle;        {Handle to a file system}
  HF  = THandle;        {Handle to a file system baggage file}


{FPtrs is an array of pointers to functions.}
  pFPtrs = ^FPtrs;
  FPtrs = Array [HE_NotUsed..HE_RcDestroyFileSys] of Pointer;


{Equivalent to our "Reset" function}
  LPFN_HfsOpenSz = Function(sz: PChar;		{Name of helpfile system}
							bFlags: Byte	{If 0 then is read/write, if 2 then readonly}
							): HFS;			{returns a handle to the filesystem}
							{$IFDEF Win32} StdCall;{$ENDIF}
{Equivalent to our "Close" function}
  LPFN_RcCloseHfs = Function(fs: HFS 		{Handle to the filesystem}
							): RC;          {returns error code (e.g. rcSuccess...see above}
							{$IFDEF Win32} StdCall;{$ENDIF}

{OpenFile equiv for Help internal filesystem}
  LPFN_HfOpenHfs = Function(fs: HFS;		{Handle to the filesystem}
							szName: PChar;	{Name of specific file, uppercase}
							bFlags: Byte    {If 0 then is read/write, if 2 then readonly}
							): HF;          {returns handle to the file itself}
							{$IFDEF Win32} StdCall;{$ENDIF}

{CloseFile equivalent for Help IFS}
  LPFN_RcCloseHf = Function(f:HF			{handle of opened file}
							): RC;			{returns the error code value}
							{$IFDEF Win32} StdCall;{$ENDIF}

{BlockRead for Help IFS}
  LPFN_LcbReadHf = Function(f:HF;			{handle of the opened file}
							Var qb;			{variable pointer to the buffer which receives data,
											 indicates current position in the pointer for
											 blockread operations}
							lcb: Longint	{pointer size in bytes}
							): Longint;     {returns number of bytes read into the buffer}
							{$IFDEF Win32} StdCall;{$ENDIF}

{Retrieve file's position in IFS}
  LPFN_LTellHf = Function(f: HF				{handle to the opened file}
							): Longint;     {returns the file's offset in the IFS}
							{$IFDEF Win32} StdCall;{$ENDIF}
{FileSeek for Help IFS}
  LPFN_LSeekHf = Function(f: HF;			{handle to the opened file}
						  lOffset: Longint; {offset from beginning of file}
						  wOrigin: Word     {seek mode: 0:from start,
														1: From current pos.
														2: From end}
						  ): Longint;		{returns position}
						  {$IFDEF Win32} StdCall;{$ENDIF}

{Determine end-of-file bool for reading HFS}
  LPFN_FEofHf = Function(f: HF): 			{handle to the opened file}
						BOOL; 				{returns TRUE if EOF reached}
						{$IFDEF Win32} StdCall;{$ENDIF}

{Retrieve size of current open file}
  LPFN_LcbSizeHf = Function(f: HF): 		{handle to the opened file}
							Longint;		{size of the file}
							{$IFDEF Win32} StdCall;{$ENDIF}

{Determine access permissions for components of the helpfile's IFS}
  LPFN_FAccessHfs = Function(fs: HFS;		{handle of the open filesystem}
							 szName: PChar;	{name of the file queried}
							 bFlags: Byte): {If 0 then R/W, if 2 then ReadOnly}
							 BOOL;			{File available for requested access method}
							 {$IFDEF Win32} StdCall;{$ENDIF}

{Retrieve last error by RC value}
  LPFN_ErrorW = Procedure(nError: Integer); {Retrieve last error code}
							{$IFDEF Win32} StdCall;{$ENDIF}

{Retrieve last error by string}
  LPFN_ErrorSz = Procedure(lpstr: PChar);	{Retrieve last error *string*?}
							{$IFDEF Win32} StdCall;{$ENDIF}

{Retrieve specific info about the helpfile.
 NOTE: This function was NOT correctly defined in MS' own docs.
 Parameter order was reversed in the HAG documentation.}
  LPFN_LGetInfo = Function(wItem: Cardinal;	{Info to retrieve. List:
											{ $01: hInstance of this WinHelp}
											{ $02 HWnd of main window}
											{ $03 HWnd of window making call}
											{ $04 Handle to this helpfile's IFS}
											{ $05 Foreground color}
											{ $06 Background color}
											{ $07 actual topic index number}                                     { $08 Speicher-Handle    }
											{ $08 Pointer to qchPath for this helpfile}
											{ $64 Special handle for unknown}
											{ $65 ???}
											{ $66 ???}
											{ $67 PChar auf "MAIN"/"SECONDARY"}
											{ $68 Makro in Bearbeitung ($0000/$0001)}
							wnd: Hwnd		{windowhandle to query}
						   ): Longint;		{address of info returned or actual value}
							{$IFDEF Win32} StdCall;{$ENDIF}

{This is equivalent to a WinHelp call done internally}
  LPFN_FAPI = Function(qchHelp: PChar;		{qchPath of helpfile}
					   wCommand: Word;		{HELP_[command]}
					   ulData: Longint		{Pointer to string (HELP_JUMP etc)
											 or actual value to pass (HELP_CONTEXT etc}
					   ): Longint;			{Error info structure; usually treated as bool}
						{$IFDEF Win32} StdCall;{$ENDIF}

{Retrieve low-level info about open file}
  LPFN_RcLLInfoFromHf = Function(f: HF;
								 wOption: Word;
								 Var qfid: Word;
								 Var qlBase,qlcb:
								 Longint): RC;
								{$IFDEF Win32} StdCall;{$ENDIF}

{Retrieve data about a specific baggage file}
  LPFN_RcLLInfoFromHfs=Function(fs:HFS;     {handle to current filesystem}
					   szName: PChar;		{name of file, uppercase}
					   wOption: Word;		{0/1/2}
					   Var qfid: Word;		{file ID}
					   Var qlBase: Longint;	{offset of file in filesystem}
					   qlcb: Longint): RC;	{returns RC error code}
				   {$IFDEF Win32} StdCall;{$ENDIF}

{Total size of IFS}
  LPFN_LcbSizeHfs = Function(fs: HFS		{handle to current filesystem}
							): Longint;		{total size of filesystem}
						{$IFDEF Win32} StdCall;{$ENDIF}

{Create a new baggage file}
  LPFN_HfCreateFileHfs = Function(fs: HFS;	{handle to current filesystem}
						  szName: PChar;	{create new baggage file}
						  wFlags: Word		{creation mode (??)}
						  ): Word;			{wErr error code}
					{$IFDEF Win32} StdCall;{$ENDIF}

{Erase a baggage file}
  LPFN_RcUnlinkFileHfs = Function(fs : HFS;	{handle to current filesystem}
						  szName: PChar		{name of file to erase}
						  ): Word;	{wErr error code}
								{$IFDEF Win32} StdCall;{$ENDIF}

{}
  LPFN_RcFlushHf = Function(f: HF			{handle to baggage file}
							): Word;
						{$IFDEF Win32}StdCall;{$ENDIF}

{write data to a baggaged file}
  LPFN_LcbWriteHf = Function(f: HF;         {handle to baggage file}
							 Var qb;        {address of buffer with data to write}
							 lcb: Longint	{length of the buffer}
							 ): Longint;	{num bytes written}
							{$IFDEF Win32} StdCall;{$ENDIF}

{Rename a baggage file}
  LPFN_RcRenameFileHfs = Function(fs: HFS;	{handle to current filesystem}
						 szOldName: PChar;	{old name of file}
						 szNewName: PChar	{new name of file}
						 ): Word;           {wErr error code}
					{$IFDEF Win32} StdCall;{$ENDIF}

{Dump the current file}
  LPFN_RcAbandonHf = Function(fs: HFS		{handle to current filesystem}
							  ): Word;		{wErr error code}
					{$IFDEF Win32} StdCall;{$ENDIF}

  LPFN_HfsCreateFileSys = Function(szName: PChar; {name (??)}
								   Var pb	{pointer to parameter block}
								   ) : Word;{wErr error code}
							{$IFDEF Win32} StdCall;{$ENDIF}

  LPFN_RcDestroyFileSys = Function(szName: PChar {name (??)}
								   ): Word;{wErr error code}
							{$IFDEF Win32} StdCall;{$ENDIF}

{Pointers to internal functions. These are accessible to DLLs that Use this unit.}
Var
  HfsOpenSz       : LPFN_HfsOpenSz;
  RcCloseHfs      : LPFN_RcCloseHfs;
  HfOpenHfs       : LPFN_HfOpenHfs;
  RcCloseHf       : LPFN_RcCloseHf;
  LcbReadHf       : LPFN_LcbReadHf;
  LTellHf         : LPFN_LTellHf;
  LSeekHf         : LPFN_LSeekHf;
  FEofHf          : LPFN_FEofHf;
  LcbSizeHf       : LPFN_LcbSizeHf;
  FAccessHfs      : LPFN_FAccessHfs;
  RcLLInfoFromHf  : LPFN_RcLLInfoFromHf;
  RcLLInfoFromHfs : LPFN_RcLLInfoFromHfs;
  ErrorW          : LPFN_ErrorW;
  ErrorSz         : LPFN_ErrorSz;
  LGetInfo        : LPFN_LGetInfo;
  FAPI            : LPFN_FAPI;
  LcbSizeHfs      : LPFN_LcbSizeHfs;
  HfCreateFileHfs : LPFN_HfCreateFileHfs;
  RcUnlinkFileHfs : LPFN_RcUnlinkFileHfs;
  RcFlushHf       : LPFN_RcFlushHf;
  LcbWriteHf      : LPFN_LcbWriteHf;
  RcRenameFileHfs : LPFN_RcRenameFileHfs;
  RcAbandonHf     : LPFN_RcAbandonHf;
  HfsCreateFileSys : LPFN_HfsCreateFileSys;
  RcDestroyFileSys : LPFN_RcDestroyFileSys;

Function GetCallBacks (vptr : pFPtrs; lVersion : Longint) : Longint;

(*****************************************************************************)

Implementation

{GetCallBacks -- Copy function pointers from array to global function pointers.
 This declaration permits relatively simple calling of functions (see the baggage
 function for an example). Allows us to call "FAPI(...)" instead of "LPFN_FAPI(vptr^[HE_API])"}

Function GetCallBacks (vptr : pFPtrs; lVersion : Longint) : Longint;
Begin
  HfsOpenSz       := LPFN_HfsOpenSz  (vptr^[HE_HfsOpenSz]);
  RcCloseHfs      := LPFN_RcCloseHfs (vptr^[HE_RcCloseHfs]);
  HfOpenHfs       := LPFN_HfOpenHfs  (vptr^[HE_HfOpenHfs]);
  RcCloseHf       := LPFN_RcCloseHf  (vptr^[HE_RcCloseHf]);
  LcbReadHf       := LPFN_LcbReadHf  (vptr^[HE_LcbReadHf]);
  LTellHf         := LPFN_LTellHf    (vptr^[HE_LTellHf]);
  LSeekHf         := LPFN_LSeekHf    (vptr^[HE_LSeekHf]);
  FEofHf          := LPFN_FEofHf     (vptr^[HE_FEofHf]);
  LcbSizeHf       := LPFN_LcbSizeHf  (vptr^[HE_LcbSizeHf]);
  FAccessHfs      := LPFN_FAccessHfs (vptr^[HE_FAccessHfs]);
  RcLLInfoFromHf  := LPFN_RcLLInfoFromHf  (vptr^[HE_RcLLInfoFromHf]);
  RcLLInfoFromHfs := LPFN_RcLLInfoFromHfs (vptr^[HE_RcLLInfoFromHfs]);
  ErrorW          := LPFN_ErrorW     (vptr^[HE_ErrorW]);
  ErrorSz         := LPFN_ErrorSz    (vptr^[HE_ErrorSz]);
  LGetInfo        := LPFN_LGetInfo   (vptr^[HE_GetInfo]);
  FAPI            := LPFN_FAPI       (vptr^[HE_API]);
  LcbSizeHfs      := LPFN_LcbSizeHfs (vptr^[HE_LcbSizeHfs]);
  HfCreateFileHfs := LPFN_HfCreateFileHfs (vptr^[HE_HfCreateFileHfs]);
  RcUnlinkFileHfs := LPFN_RcUnlinkFileHfs (vptr^[HE_RcUnlinkFileHfs]);
  RcFlushHf       := LPFN_RcFlushHf  (vptr^[HE_RcFlushHf]);
  LcbWriteHf      := LPFN_LcbWriteHf (vptr^[HE_LcbWriteHf]);
  RcRenameFileHfs := LPFN_RcRenameFileHfs (vptr^[HE_RcRenameFileHfs]);
  RcAbandonHf     := LPFN_RcAbandonHf (vptr^[HE_RcAbandonHf]);
  HfsCreateFileSys := LPFN_HfsCreateFileSys (vptr^[HE_HfsCreateFileSys]);
  RcDestroyFileSys := LPFN_RcDestroyFileSys (vptr^[HE_RcDestroyFileSys]);
  GetCallBacks := 1;
End;

End.
